import { Http, IEventParam, Metadata, Singleton } from '@zeedhi/core';
import { Menu, MenuButton, AlertService } from '@zeedhi/common';
import { TekLibAuth } from '@zeedhi/tek-lib';
import { TekUserInfo } from '@zeedhi/teknisa-components-common';

@Singleton
export class AppController {
  public hasImage: boolean = false;

  get notHasImage(): boolean {
    return !this.hasImage;
  }

  public imagePath: string = '';

  public microservicePage: boolean = false;

  public toggleMenu({ component }: IEventParam<MenuButton>) {
    const menu = Metadata.getInstance<Menu>(component.menuName);
    menu.toggleMenu();
    if (menu.opened && !menu.miniState) {
      if (menu.showSearch) {
        setTimeout(() => {
          const searchInput = document.querySelector('.zd-menu-search-input input');
          if (searchInput) (searchInput as HTMLElement).focus();
        }, 100);
      } else {
        const firstMenuItem = document.querySelector('.zd-menu-link, .zd-menu-group > div');
        if (firstMenuItem) (firstMenuItem as HTMLElement).focus();
      }
    }
  }

  get userName() {
    const username = localStorage.getItem('LOGIN_USER_NAME') || '';
    return this.getFormattedName(username);
  }

  get nameInitialLetter() {
    const name = this.userName;
    return name.slice(0, 1);
  }

  get logo() {
    return '/assets/img/tecfood.svg';
  }

  public showMessageExpiredMenu() {
    TekLibAuth.showMessageExpiredMenu();
  }

  public closeMenu({ component }: IEventParam<MenuButton>) {
    const menu = Metadata.getInstance<Menu>(component.menuName);
    if (menu.opened && !menu.miniState) {
      menu.toggleMenu();
    }
  }

  public escAction() {
    AlertService.hide();
  }

  public async userInfoOnCreated() {
    const userInfoComponent = Metadata.getInstance<TekUserInfo>('userInfo');
    const currentModule = this.getCurrentModule();

    const { data } = await Http.get('/lib_generalInfo', { params: { currentModule } });
    userInfoComponent.versionInfo = data.versionInfo;

    if (data.userInfo.IMAGE) {
      this.hasImage = true;
      this.imagePath = data.userInfo.IMAGE;
    } else {
      this.hasImage = false;
    }
  }

  public userInfoOnLogoutClick() {
    TekLibAuth.logoutConfirm();
  }

  private getFormattedName(username: string): string {
    const usernameLowerCase = username.toLowerCase();
    const name = usernameLowerCase.replace(/(^\w{1})|(\s+\w{1})/g, (letter: string) => letter.toUpperCase());
    const splitName = name.split(' ');
    if (splitName.length > 1) {
      const firstName = splitName.shift();
      const lastName = splitName.pop();
      let middleName = '';

      splitName.forEach((item) => {
        middleName += ` ${item.slice(0, 1)}. `;
      });

      return `${firstName} ${middleName} ${lastName}`;
    }

    return name;
  }

  private getCurrentModule(): string {
    const { href } = window.location;
    let currentModule = 'product';
    if (href.includes('/modules/')) {
      let auxHref = href.split('modules/');
      auxHref = auxHref[1].split('/');
      [currentModule] = auxHref;
    }

    return currentModule;
  }
}
