<?php

namespace DsbNext\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;

class DashboardController {

  protected $crudService;
  
  public function __construct($connection, $environment, $util, $crudService) {
    $this->crudService	= $crudService;
    $this->connection   = $connection;
    $this->environment  = $environment;
    $this->util         = $util;
  }
  
  public function visibleIndicators(Request\Filter $request, Response $response) {
    $params = array(
      "CDOPERADOR" => $this->environment->getCdOperador(),
      "CDFILIAL" => $this->environment->getCdFilial(),
      "NRORG" => $this->environment->getNrorg()
    );
    $params = ['CDOPERADOR','CDFILIAL','NRORG'];
    $conditions = $this->util->prepareConditions($params);
    $data = $this->crudService->getFilteredDataSet('visibleIndicators', $conditions)->getRows();
    $response->addDataSet(new DataSet("visibleIndicators", array_column($data,'NRINDICADORTF')));
  }

  public function panelConfig(Request\Filter $request, Response $response) {
    $params = ['CDOPERADOR', 'NRORG'];
    $conditions = $this->util->prepareConditions($params);
      
    $data = $this->crudService->getFilteredDataSet('panelConfig', $conditions)->getRows();
	  $response->addDataSet(new DataSet("panelConfig", $data));
  }
  
  public function changeOrderDash(Request\DataSet $request, Response $response) {
    $panel = $request->getParameters()['selectedPanel'];
    $ordem = $request->getParameters()['orderIndicDash'];
    
    $params = array(
      'CDOPERADOR'         => $this->environment->getCdOperador(),
      'NRORG'              => $this->environment->getNrorg(),
      'ORDER'              => $ordem,
      'PANEL'              => $panel
    );
    
  	try {
      $stmt = $this->connection->prepare(
  	    "DECLARE V_NRPAINELORDERINDIC NUMBER;
  	     BEGIN
  	        UPDATE PAINELORDERINDIC
  	           SET DASHGU = CASE WHEN :PANEL = 'dashGU' THEN :ORDER ELSE DASHGU END,
  	               DASHGO = CASE WHEN :PANEL = 'dashGO' THEN :ORDER ELSE DASHGO END,
  	               DASHTD = CASE WHEN :PANEL = 'dashTD' THEN :ORDER ELSE DASHTD END,
  	               DASHTA = CASE WHEN :PANEL = 'dashTA' THEN :ORDER ELSE DASHTA END
  	         WHERE CDOPERADOR = :CDOPERADOR
  	           AND NRORG = :NRORG;
  	           
  	        IF SQL%ROWCOUNT = 0 THEN
               NOVO_CODIGO('PAINELORDERINDIC', V_NRPAINELORDERINDIC);
              
  	           INSERT INTO PAINELORDERINDIC 
  	             (NRPAINELORDERINDIC, CDOPERADOR,
  	              DASHGU,
  	              DASHGO, 
  	              DASHTD,
  	              DASHTA, NRORG)
  	           VALUES
  	             (V_NRPAINELORDERINDIC, :CDOPERADOR,
  	             CASE WHEN :PANEL = 'dashGU' THEN :ORDER ELSE NULL END, 
  	             CASE WHEN :PANEL = 'dashGO' THEN :ORDER ELSE NULL END,
  	             CASE WHEN :PANEL = 'dashTD' THEN :ORDER ELSE NULL END,
  	             CASE WHEN :PANEL = 'dashTA' THEN :ORDER ELSE NULL END, :NRORG);
  	        END IF;
  	        
  	        COMMIT;
  	     END;"
    	);
      $stmt->execute($params);
    } catch (\Exception $ex) {
      var_dump($ex->getMessage());
    }
  }
}
