import { Http, Metadata } from '@zeedhi/core';
import { Indicators } from '@/helper/Indicators';
import SharedInfo from '@/helper/SharedInfo';
import { ZdDashboard, ZdGrid, ZdButtonGroup } from '@zeedhi/vuetify';
import { Formatting } from '@/util/Utilities';
import { GenericValueController } from '@/controllers/GenericValueController';
import ChangeLabel from '@/helper/Labels';

export class DashboardController {
  private _accessIndicators: number[];

  public editingModeDash = false;
 
  public get accessIndicators(): number[] {
    /* eslint no-underscore-dangle: 0 */
    return this._accessIndicators;
  }

  public set accessIndicators(accessIndicators: number[]) {
    /* eslint no-underscore-dangle: 0 */
    this._accessIndicators = accessIndicators;
  }
  
  public getDataGrid({ component }) {
    const formatting = new Formatting();
    const genericValues = new GenericValueController();
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    const routeName = component.children[0].datasourceName;
    const sendObject = {
      filter: [],
    };
    let dayOfWeekProc;
    let data;
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        /* eslint no-console: ["error", { allow: ["warn", "error", "info", "log"] }] */
        data = response.data.dataset[routeName].map((obj) => {
          Object.keys(obj).forEach((key) => {
            if (typeof obj[key] === 'number') {
              if (key === 'NRDTPROC') {
                dayOfWeekProc = obj[key];
              } else if (key !== 'NRDIA') {
                obj[key] = formatting.formatValue('float', obj[key]);
              }
            } else if (obj[key] instanceof Date) {
              console.log(obj[key]);
            } else if (key === 'NMITEM') {
              obj[key] = ChangeLabel.getLabel(obj[key]);
            }
            if (key === 'NRDTPROC') {
              dayOfWeekProc = obj[key];
            }
          });
        
          return obj;
        });
        
        component.datasource.updateData(data);
        if (routeName === 'todayDeliveries') {
          const zdgroupbutton = Metadata.getInstance<ZdButtonGroup>('daysWeek');
          
          zdgroupbutton.selectedButtons = dayOfWeekProc - 1;
          component.datasource.setFilter({
            NRDIA: dayOfWeekProc,
          });
        }
      });
    } else {
      data = genericValues.valueGrid[component.name].map((obj) => {
        Object.keys(obj).forEach((key) => {
          if (typeof obj[key] === 'number') {
            if (key === 'NRDTPROC') {
              dayOfWeekProc = obj[key];
            } else if (key !== 'NRDIA') {
              obj[key] = formatting.formatValue('float', obj[key]);
            }
          } else if (obj[key] instanceof Date) {
            console.log(obj[key]);
          } else if (key === 'NMITEM') {
            obj[key] = ChangeLabel.getLabel(obj[key]);
          }
        });
      
        return obj;
      });
      
      component.datasource.updateData(data);
      if (routeName === 'todayDeliveries') {
        const zdgroupbutton = Metadata.getInstance<ZdButtonGroup>('daysWeek');
        
        zdgroupbutton.selectedButtons = dayOfWeekProc - 1;
        component.datasource.setFilter({
          NRDIA: dayOfWeekProc,
        });
      }
    }
  }
  
  public async cardList({ component }) {
    component.cards = [];
    // await this.accessibleIndicators();
    await SharedInfo.waitMyBrother();
    const indic = new Indicators();
    const visible = this.accessIndicators;
    let visibleCards = [];
    try {
      switch (component.name) {
        case 'dashGU': visibleCards = this.sortCards(indic.cardListGU.filter((obj) => (
          visible.includes(obj.cardId)
        )), component.name);
          break;
        case 'dashGO': visibleCards = this.sortCards(indic.cardListGO.filter((obj) => (
          visible.includes(obj.cardId)
        )), component.name);
          break;
        case 'dashAC': visibleCards = this.sortCards(indic.cardListTD.filter((obj) => (
          visible.includes(obj.cardId)
        )), component.name);
          break;
        case 'dashTA': visibleCards = this.sortCards(indic.cardListTA.filter((obj) => (
          visible.includes(obj.cardId)
        )), component.name);
          break;
        default: visibleCards = [];
      }

      component.cards = visibleCards;
    } catch (error) {
      // Adicionar mensagem de erro
      component.cards = [];
    }
  }
  
  public async accessibleIndicators() {
    const indic = new Indicators();
    const sendObject = {
      filter: [],
    };
    
    try {
      if (typeof this.accessIndicators === 'undefined') {
        const routeName = 'visibleIndicators';
        const response = await Http.post('/visibleIndicators', sendObject);
        this.accessIndicators = response.data.dataset[routeName].concat(indic.auxDash);
      }
    } catch (error) {
      // Adicionar mensagem de erro
    }
  }
  
  public async visiblePanel({ component }) {
    await this.accessibleIndicators();
    const indic = new Indicators();
    
    let panelVisible = false;
    let dashVisible = false;
    
    component.tabs.forEach((obj) => {
      switch (obj.name) {
        case 'unitManager': panelVisible = indic.cardsGU.some((item) => this.accessIndicators.includes(item));
          break;
        case 'operationalManager': panelVisible = indic.cardsGO.some((item) => this.accessIndicators.includes(item));
          break;
        case 'costAnalysis': panelVisible = indic.cardsTD.some((item) => this.accessIndicators.includes(item));
          break;
        case 'tracking': panelVisible = indic.cardsTA.some((item) => this.accessIndicators.includes(item));
          break;
        default: panelVisible = false;
      }
      obj.isVisible = panelVisible;
      dashVisible = dashVisible || panelVisible;
    });
    
    component.isVisible = dashVisible;
  }
  
  public cancelEditOrder() {
    this.editingModeDash = false;
    window.location.reload();
  }
  
  public confirmEditOrder(components) {
    let panel;
    switch (components.component.name) {
      case 'confirmEditOrderGU': panel = 'dashGU';
        break;
      default: panel = '';
    }
    if (panel !== '') {
      const dash = Metadata.getInstance<ZdDashboard>(panel);
      const newOrder = dash.cards.map((obj) => obj.cardId).join(',');
      const sendObject = {
        orderIndicDash: newOrder,
        selectedPanel: panel,
      };
  
      Http.post('/changeOrderDash', sendObject).then((response) => {
        console.log(response);
      });
    }
    this.editingModeDash = false;
  }
  
  sortCards(cards: any, panel: string): any {
    const cardOrder = SharedInfo.getOrderPanel(panel);
    
    if (cardOrder) {
      const orderIdCard = cardOrder.split(',').map(Number);
      
      return cards.sort((item, itemAux) => {
        const index = orderIdCard.indexOf(item.cardId);
        const indexAux = orderIdCard.indexOf(itemAux.cardId);
        
        if (index === -1 && indexAux === -1) {
          return 0;
        }
        if (index === -1) {
          return 1;
        }
        if (indexAux === -1) {
          return -1;
        }
  
        return index - indexAux;
      });
    }
    return cards;
  }

  public handlingCards() {
    this.editingModeDash = true;
  }
  
  get editingMode() {
    return this.editingModeDash;
  }
  
  public showDescription() {
    
  }
  
  public async panelConfig() {
    if (SharedInfo.getSingleRun) {
      await this.accessibleIndicators();
      let singleRun = true;
      const sendObject = {
        filter: [],
      };

      try {
        const response = await Http.post('/panelConfig', sendObject);
        const panelConfig = response.data.dataset.panelConfig;
      
        panelConfig.forEach((obj) => {
          SharedInfo.setLimit(obj.NRINDICADORTF, obj.VRLIMMETA, obj.VRLIMMINIMO);
          
          if (singleRun) {
            SharedInfo.setOrderPanel(obj.loaded, obj.DASHGU, obj.DASHGO, obj.DASHTD, obj.DASHTA);
            singleRun = false;
          }
        });
      
        if (SharedInfo.loadingSolve) {
          SharedInfo.loadingSolve();
        }
      } catch (error) {
        console.log('teste');
      }
    }
  }
  
  public daysWeek(components) {
    const todayDeliveriesGrid = Metadata.getInstance<ZdGrid>('todayDeliveriesGrid');
    let nrDay;
    
    switch (components.component.name) {
      case 'DOM': nrDay = 1;
        break;
      case 'SEG': nrDay = 2;
        break;
      case 'TER': nrDay = 3;
        break;
      case 'QUA': nrDay = 4;
        break;
      case 'QUI': nrDay = 5;
        break;
      case 'SEX': nrDay = 6;
        break;
      default: nrDay = 7;
    }
    
    todayDeliveriesGrid.datasource.setFilter({
      NRDIA: nrDay,
    });
  }
}
